﻿Type=Class
Version=3.82
B4A=true
@EndOfDesignText@
'Class module
Public Sub Class_Globals
	Dim uuid As String
	Type iconDetail( mimetype As String, _
					 width As Int, _
					 url As String)	
End Sub

'Creates an XOM document 
'XML - A valid XML string
'serverId - uuid of server that XML file came from
'
'Updates the server details in UPNP.serverID map
'Example: <code>
'Dim descParser As Desc_Parser
'descParser.Initialize(job.GetString, jobId)</code>
Public Sub Initialize(XML As String, serverId As String)
	uuid = serverId
	' Initialise and build XOM document
	Dim XOM As XOMBuilder
	XOM.Initialize("XOM")
	XOM.BuildFromString(XML, "", Null)
End Sub

Private Sub XOM_BuildDone(XOM_Doc As XOMDocument, Tag As Object)
	' Event raised when XOM document completed or failed
	If XOM_Doc.IsInitialized Then
		' Get root node and locate device info
		Dim rootNode As XOMElement = XOM_Doc.RootElement
		GetDeviceInfo(rootNode)
	Else
		Debug.debugLog("XOM_Doc is not initialized: " & LastException) 'DEBUG
	End If
End Sub

#Region -------------------------[ Basic Device Info ]-------------------------
Private Sub GetDeviceInfo(parentNode As XOMElement)
	Dim server As UPNPserver = UPNP.serverID.Get(uuid)
	' Get all child nodes
	Dim childNodes As XOMElements = parentNode.GetChildElements
	Dim i As Int
	' Find the device node
	For i = 0 To childNodes.Size - 1
		If childNodes.GetElement(i).LocalName.ToLowerCase = "device" Then
			Dim deviceNodes As XOMElements = childNodes.GetElement(i).GetChildElements
			Exit
		End If
	Next
	' Find device info
	For i = 0 To deviceNodes.Size - 1
		Dim elementName, elementValue As String
		elementName = deviceNodes.GetElement(i).LocalName.ToLowerCase
		elementValue = deviceNodes.GetElement(i).Value
		' Extract only the required device info
		Select Case elementName
			Case "friendlyname"
				server.Name = elementValue
			Case "modeldescription", "modelname"
				server.modelDescription = elementValue
			Case "iconlist"
				Dim iconNodes As XOMElements = deviceNodes.GetElement(i).GetChildElements
				server.iconURL = Format.CheckLeadingSlash( GetIcon(iconNodes) )
				CallSub3(UPNP, "Request_Icon", server.rootURL & server.iconURL, uuid)
			Case "servicelist"
				Dim serviceDetails As List
				Dim serviceNodes As XOMElements = deviceNodes.GetElement(i).GetChildElements
				' Get connection service details
				serviceDetails.Initialize2( GetService("connection", serviceNodes) )
				server.connectionType = serviceDetails.Get(0)
				server.connectionSCPDURL = Format.CheckLeadingSlash( serviceDetails.Get(1) )
				server.connectionControlURL = Format.CheckLeadingSlash( serviceDetails.Get(2) )
				server.connectionEventSubURL = Format.CheckLeadingSlash( serviceDetails.Get(3) )
				' Get content service details
				serviceDetails.Initialize2( GetService("content", serviceNodes) )
				server.contentType = serviceDetails.Get(0)
				server.contentSCPDURL = Format.CheckLeadingSlash( serviceDetails.Get(1) )
				server.contentControlURL = Format.CheckLeadingSlash( serviceDetails.Get(2) )
				server.contentEventSubURL = Format.CheckLeadingSlash( serviceDetails.Get(3) )
		End Select
	Next
End Sub
#End Region

#Region -------------------------[ Device Icon ]-------------------------
Private Sub GetIcon(iconNodes As XOMElements) As String
	Dim iconList As List
	iconList.Initialize
	Dim pngIcon, jpgIcon, bmpIcon As Boolean=False
	Dim i As Int
	For i = 0 To iconNodes.Size - 1
		' Find the device node
		If iconNodes.GetElement(i).LocalName.ToLowerCase = "icon" Then
			Dim iconNode As XOMElements = iconNodes.GetElement(i).GetChildElements
			' Get all icons listed in XML device description
			Dim x As Int
			Dim icon As iconDetail
			For x = 0 To iconNode.Size - 1
				Dim elementName, elementValue As String
				elementName = iconNode.GetElement(x).LocalName.ToLowerCase
				elementValue = iconNode.GetElement(x).Value
				Select Case elementName
					Case "mimetype"
						icon.mimetype = elementValue
					Case "width"
						icon.width = elementValue
					Case "url"
						icon.url = elementValue
				End Select			
				' Determine which file types are present
				If icon.mimetype.Contains("png") Then pngIcon = True
				If icon.mimetype.Contains("jpeg") Then jpgIcon = True
				If icon.mimetype.Contains("bmp") Then bmpIcon = True
			Next
			iconList.Add(icon)
		End If
	Next
	' Filter out png as the prefered file type, after that try jpeg, then finally bmp
	For i = iconList.Size - 1 To 0 Step - 1
		' Get details of each icon
		Dim icon As iconDetail
		icon = iconList.Get(i)
		' Remove least desired file types
		If pngIcon = True Then
			If icon.mimetype.Contains("png") = False Then iconList.RemoveAt(i)
		Else If jpgIcon = True Then
			If icon.mimetype.Contains("jpeg") = False Then iconList.RemoveAt(i)
		Else If bmpIcon = True Then
			If icon.mimetype.Contains("bmp") = False Then iconList.RemoveAt(i)
		End If
	Next
	' Get the icon file closest to the desired size
	iconList.SortType("width", True)
	For i = 0 To iconList.Size - 1
		Dim icon As iconDetail
		icon = iconList.Get(i)
		If icon.width >= 75 Then Exit
	Next
	Return icon.url
End Sub
#End Region

#Region -------------------------[ Device Service Details ]-------------------------
Private Sub GetService(requiredService As String, serviceNodes As XOMElements) As List
	Dim serviceDetails As List
	Dim serviceType, scpdURL, controlURL, eventSubURL As String
	Dim i As Int
	For i = 0 To serviceNodes.Size - 1
		' Find the service nodes
		If serviceNodes.GetElement(i).LocalName.ToLowerCase = "service" Then
			Dim serviceNode As XOMElements = serviceNodes.GetElement(i).GetChildElements
			' Get all service info listed in XML device description
			Dim x As Int
			Dim elementName, elementValue As String
			For x = 0 To serviceNode.Size - 1
				elementName = serviceNode.GetElement(x).LocalName.ToLowerCase
				elementValue = serviceNode.GetElement(x).Value
				Select Case elementName
					Case "servicetype"
						serviceType = elementValue
					Case "scpdurl"
						scpdURL = elementValue
					Case "controlurl"
						controlURL = elementValue
					Case "eventsuburl"
						eventSubURL = elementValue
				End Select
			Next
			' See if the service info is what is required
			If serviceType.ToLowerCase.Contains(requiredService) Then
				serviceDetails.Initialize
				serviceDetails.Add(serviceType)
				serviceDetails.Add(scpdURL)
				serviceDetails.Add(controlURL)
				serviceDetails.Add(eventSubURL)
				Return serviceDetails
			End If
		End If
	Next
	' If required service not found return an empty list
	serviceDetails.Initialize
	serviceDetails.Add("Not Found")
	Return serviceDetails
End Sub
#End Region